@@ -4,6 +4,26 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "pinyincontext.h"
+#include <algorithm>
+#include <cassert>
+#include <cstddef>
+#include <functional>
+#include <iterator>
+#include <limits>
+#include <memory>
+#include <stdexcept>
+#include <string>
+#include <string_view>
+#include <unordered_set>
+#include <utility>
+#include <vector>
+#include <fcitx-utils/charutils.h>
+#include <fcitx-utils/inputbuffer.h>
+#include <fcitx-utils/keysym.h>
+#include <fcitx-utils/macros.h>
+#include <fcitx-utils/signals.h>
+#include <fcitx-utils/stringutils.h>
+#include <fcitx-utils/utf8.h>
 #include "libime/core/historybigram.h"
 #include "libime/core/inputbuffer.h"
 #include "libime/core/languagemodel.h"
@@ -15,26 +35,6 @@
 #include "pinyinencoder.h"
 #include "pinyinime.h"
 #include "pinyinmatchstate.h"
-#include <algorithm>
-#include <cassert>
-#include <cstddef>
-#include <fcitx-utils/charutils.h>
-#include <fcitx-utils/inputbuffer.h>
-#include <fcitx-utils/keysym.h>
-#include <fcitx-utils/macros.h>
-#include <fcitx-utils/signals.h>
-#include <fcitx-utils/stringutils.h>
-#include <fcitx-utils/utf8.h>
-#include <functional>
-#include <iterator>
-#include <limits>
-#include <memory>
-#include <stdexcept>
-#include <string>
-#include <string_view>
-#include <unordered_set>
-#include <utility>
-#include <vector>
 
 namespace libime {
 
@@ -44,15 +44,21 @@ enum class LearnWordResult {
     Ignored,
 };
 
+enum class SelectedPinyinType {
+    Normal,
+    Custom,
+    Separator,
+};
+
 struct SelectedPinyin {
     SelectedPinyin(size_t s, WordNode word, std::string encodedPinyin,
-                   bool custom)
+                   SelectedPinyinType type)
         : offset_(s), word_(std::move(word)),
-          encodedPinyin_(std::move(encodedPinyin)), custom_(custom) {}
+          encodedPinyin_(std::move(encodedPinyin)), type_(type) {}
     size_t offset_;
     WordNode word_;
     std::string encodedPinyin_;
-    bool custom_;
+    SelectedPinyinType type_;
 };
 
 class PinyinContextPrivate : public fcitx::QPtrHolder<PinyinContext> {
@@ -131,7 +137,7 @@ public:
                     continue;
                 }
                 auto text = candidate.toString();
-                if (candidatesToCursorSet_.count(text)) {
+                if (candidatesToCursorSet_.contains(text)) {
                     continue;
                 }
                 candidatesToCursor_.push_back(candidate);
@@ -147,7 +153,7 @@ public:
                     SentenceResult partial(newSentence,
                                            newSentence.back()->score());
                     auto text = partial.toString();
-                    if (candidatesToCursorSet_.count(text)) {
+                    if (candidatesToCursorSet_.contains(text)) {
                         continue;
                     }
                     candidatesToCursor_.push_back(partial);
@@ -170,7 +176,8 @@ public:
         if (!remain.empty()) {
             if (std::all_of(remain.begin(), remain.end(),
                             [](char c) { return c == '\''; })) {
-                selection.emplace_back(q->size(), WordNode("", 0), "", false);
+                selection.emplace_back(q->size(), WordNode("", 0), "",
+                                       SelectedPinyinType::Separator);
             }
         }
 
@@ -180,16 +187,16 @@ public:
     void select(const SentenceResult &sentence) {
         FCITX_Q();
         auto offset = q->selectedLength();
-        selectHelper([offset, &sentence,
-                      this](std::vector<SelectedPinyin> &selection) {
-            for (const auto &p : sentence.sentence()) {
-                selection.emplace_back(
-                    offset + p->to()->index(),
-                    WordNode{p->word(), ime_->model()->index(p->word())},
-                    static_cast<const PinyinLatticeNode *>(p)->encodedPinyin(),
-                    false);
-            }
-        });
+        selectHelper(
+            [offset, &sentence, this](std::vector<SelectedPinyin> &selection) {
+                for (const auto &p : sentence.sentence()) {
+                    selection.emplace_back(
+                        offset + p->to()->index(),
+                        WordNode{p->word(), ime_->model()->index(p->word())},
+                        p->as<PinyinLatticeNode>().encodedPinyin(),
+                        SelectedPinyinType::Normal);
+                }
+            });
     }
 
     void selectCustom(size_t inputLength, std::string_view segment,
@@ -199,9 +206,9 @@ public:
         selectHelper([this, offset, &segment, inputLength,
                       &encodedPinyin](std::vector<SelectedPinyin> &selection) {
             auto index = ime_->model()->index(segment);
-            selection.emplace_back(offset + inputLength,
-                                   WordNode{segment, index},
-                                   std::string(encodedPinyin), true);
+            selection.emplace_back(
+                offset + inputLength, WordNode{segment, index},
+                std::string(encodedPinyin), SelectedPinyinType::Custom);
         });
     }
 
@@ -223,14 +230,14 @@ public:
         for (auto &s : selected_) {
             isAllSingleWord =
                 isAllSingleWord &&
-                (s.empty() ||
-                 (s.size() == 1 && (s[0].word_.word().empty() ||
-                                    s[0].encodedPinyin_.size() == 2)));
+                (s.empty() || (s.size() == 1 &&
+                               (s[0].type_ == SelectedPinyinType::Separator ||
+                                s[0].encodedPinyin_.size() == 2)));
             for (auto &item : s) {
-                if (item.word_.word().empty()) {
+                if (item.type_ == SelectedPinyinType::Separator) {
                     continue;
                 }
-                if (item.custom_) {
+                if (item.type_ == SelectedPinyinType::Custom) {
                     hasCustom = true;
                 }
                 // We can't learn non pinyin word.
@@ -246,7 +253,7 @@ public:
         }
         for (auto &s : selected_) {
             for (auto &item : s) {
-                if (item.word_.word().empty()) {
+                if (item.type_ == SelectedPinyinType::Separator) {
                     continue;
                 }
                 assert(!item.encodedPinyin_.empty());
@@ -584,6 +591,8 @@ void PinyinContext::update() {
                                    d->ime_->frameSize(), &d->matchState_);
 
         d->clearCandidates();
+
+        // Add n-best result.
         for (size_t i = 0, e = d->lattice_.sentenceSize(); i < e; i++) {
             d->candidates_.push_back(d->lattice_.sentence(i));
             d->candidatesSet_.insert(d->candidates_.back().toString());
@@ -597,8 +606,10 @@ void PinyinContext::update() {
             float max = -std::numeric_limits<float>::max();
             auto distancePenalty = d->ime_->model()->unknownPenalty() /
                                    PINYIN_DISTANCE_PENALTY_FACTOR;
-            // Pull the phrase from lattice, this part is the word that's in the
-            // dict.
+
+            // Enumerate over all the lattice node, if from == bos, this is
+            // a dictionary word match.
+            // Add all words that does not contain pinyin correction.
             for (const auto &graphNode : graph.nodes(i)) {
                 auto distance = graph.distanceToEnd(graphNode);
                 auto adjust = static_cast<float>(distance) * distancePenalty;
@@ -610,7 +621,8 @@ void PinyinContext::update() {
                             min = std::min(latticeNode.score(), min);
                             max = std::max(latticeNode.score(), max);
                         }
-                        if (d->candidatesSet_.count(latticeNode.word())) {
+                        // Deduplcate.
+                        if (d->candidatesSet_.contains(latticeNode.word())) {
                             continue;
                         }
                         d->candidates_.push_back(
@@ -628,7 +640,7 @@ void PinyinContext::update() {
                     if (latticeNode.from() == bos &&
                         static_cast<const PinyinLatticeNode &>(latticeNode)
                             .isCorrection()) {
-                        if (d->candidatesSet_.count(latticeNode.word())) {
+                        if (d->candidatesSet_.contains(latticeNode.word())) {
                             continue;
                         }
                         if ((latticeNode.score() > min &&
@@ -656,7 +668,7 @@ void PinyinContext::update() {
                         !static_cast<const PinyinLatticeNode &>(latticeNode)
                              .anyCorrectionOnPath()) {
                         auto fullWord = latticeNode.fullWord();
-                        if (d->candidatesSet_.count(fullWord)) {
+                        if (d->candidatesSet_.contains(fullWord)) {
                             continue;
                         }
                         d->candidates_.push_back(
@@ -667,7 +679,34 @@ void PinyinContext::update() {
             }
         }
         std::sort(d->candidates_.begin() + beginSize, d->candidates_.end(),
-                  std::greater<SentenceResult>());
+                  std::greater<>());
+        if (const auto limit = d->ime_->wordCandidateLimit()) {
+            size_t count = 0;
+            auto &candidatesSet = d->candidatesSet_;
+            d->candidates_.erase(
+                std::remove_if(
+                    d->candidates_.begin() + beginSize, d->candidates_.end(),
+                    [&count, limit,
+                     &candidatesSet](const SentenceResult &candidate) {
+                        const bool isSinglePinyinWord =
+                            candidate.sentence().size() == 1 &&
+                            candidate.sentence()
+                                    .front()
+                                    ->as<PinyinLatticeNode>()
+                                    .encodedPinyin()
+                                    .size() == 2;
+                        if (!isSinglePinyinWord) {
+                            if (count >= limit) {
+                                candidatesSet.erase(candidate.toString());
+                                return true;
+                            }
+                            count++;
+                        }
+                        return false;
+                    }),
+                d->candidates_.end());
+        }
+
         d->candidatesToCursorNeedUpdate_ = true;
     }
 
@@ -767,8 +806,7 @@ PinyinContext::preeditWithCursor(PinyinPreeditMode mode) const {
                 std::string actualPinyin;
                 if (!syls.empty() && !syls.front().second.empty()) {
                     std::string_view candidatePinyin =
-                        static_cast<const PinyinLatticeNode *>(node)
-                            ->encodedPinyin();
+                        node->as<PinyinLatticeNode>().encodedPinyin();
                     auto nthPinyin = std::distance(node->path().begin(), iter);
                     PinyinInitial bestInitial = syls[0].first;
                     PinyinFinal bestFinal = syls[0].second[0].first;
@@ -865,7 +903,7 @@ std::vector<std::string> PinyinContext::selectedWords() const {
     std::vector<std::string> newSentence;
     for (const auto &s : d->selected_) {
         for (const auto &item : s) {
-            if (!item.word_.word().empty()) {
+            if (item.type_ != SelectedPinyinType::Separator) {
                 newSentence.push_back(item.word_.word());
             }
         }
@@ -879,7 +917,7 @@ PinyinContext::selectedWordsWithPinyin() const {
     std::vector<std::pair<std::string, std::string>> newSentence;
     for (const auto &s : d->selected_) {
         for (const auto &item : s) {
-            if (!item.word_.word().empty()) {
+            if (item.type_ != SelectedPinyinType::Separator) {
                 newSentence.emplace_back(item.word_.word(),
                                          item.encodedPinyin_);
             }
@@ -893,7 +931,7 @@ std::string PinyinContext::selectedFullPinyin() const {
     std::string pinyin;
     for (const auto &s : d->selected_) {
         for (const auto &item : s) {
-            if (!item.word_.word().empty()) {
+            if (!item.encodedPinyin_.empty()) {
                 if (!pinyin.empty()) {
                     pinyin.push_back('\'');
                 }
@@ -913,12 +951,12 @@ std::string
 PinyinContext::candidateFullPinyin(const SentenceResult &candidate) const {
     std::string pinyin;
     for (const auto &node : candidate.sentence()) {
-        if (!node->word().empty()) {
+        if (!node->as<PinyinLatticeNode>().encodedPinyin().empty()) {
             if (!pinyin.empty()) {
                 pinyin.push_back('\'');
             }
             pinyin += PinyinEncoder::decodeFullPinyin(
-                static_cast<const PinyinLatticeNode *>(node)->encodedPinyin());
+                node->as<PinyinLatticeNode>().encodedPinyin());
         }
     }
     return pinyin;
@@ -940,7 +978,7 @@ void PinyinContext::learn() {
         std::vector<std::string> newSentence;
         for (auto &s : d->selected_) {
             for (auto &item : s) {
-                if (!item.word_.word().empty()) {
+                if (item.type_ != SelectedPinyinType::Separator) {
                     // Non pinyin word. Skip it.
                     if (item.encodedPinyin_.empty()) {
                         return;
