@@ -4,20 +4,11 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "pinyinencoder.h"
-#include "libime/core/segmentgraph.h"
-#include "pinyincorrectionprofile.h"
-#include "pinyindata.h"
-#include "shuangpinprofile.h"
 #include <algorithm>
 #include <array>
-#include <boost/bimap.hpp>
-#include <boost/container/static_vector.hpp>
 #include <cassert>
 #include <cstddef>
 #include <cstdint>
-#include <fcitx-utils/charutils.h>
-#include <fcitx-utils/log.h>
-#include <fcitx-utils/stringutils.h>
 #include <functional>
 #include <initializer_list>
 #include <iterator>
@@ -30,6 +21,16 @@
 #include <unordered_map>
 #include <utility>
 #include <vector>
+#include <boost/bimap.hpp>
+#include <boost/container/static_vector.hpp>
+#include <fcitx-utils/charutils.h>
+#include <fcitx-utils/log.h>
+#include <fcitx-utils/stringutils.h>
+#include "libime/core/segmentgraph.h"
+#include "pinyincorrectionprofile.h"
+#include "pinyindata.h"
+#include "pinyindata_p.h"
+#include "shuangpinprofile.h"
 
 namespace libime {
 
@@ -295,13 +296,15 @@ PinyinEncoder::parseUserPinyin(std::string userPinyin,
                                 fuzzyFlags.test(PinyinFuzzyFlag::Inner)) ||
                                (nextPinyin.size() == 3 &&
                                 flags.test(PinyinFuzzyFlag::InnerShort))) {
-                        const auto &innerSegments = getInnerSegment();
-                        auto iter = innerSegments.find(std::string(nextPinyin));
+                        const auto &innerSegments = getInnerSegmentV2();
+                        auto iter = innerSegments.find(nextPinyin);
                         if (iter != innerSegments.end()) {
-                            result.addNext(top,
-                                           top + iter->second.first.size());
-                            result.addNext(top + iter->second.first.size(),
-                                           top + nextSize[i]);
+                            for (const auto &innerSeg : iter->second) {
+                                result.addNext(top,
+                                               top + innerSeg.first.size());
+                                result.addNext(top + innerSeg.first.size(),
+                                               top + nextSize[i]);
+                            }
                         }
                     } else if (nextPinyin.size() == 2 &&
                                flags.test(PinyinFuzzyFlag::InnerShort) &&
