@@ -4,17 +4,17 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 #include "pinyinime.h"
+#include <cstddef>
+#include <limits>
+#include <memory>
+#include <utility>
+#include <fcitx-utils/connectableobject.h>
+#include <fcitx-utils/macros.h>
 #include "libime/core/decoder.h"
 #include "libime/core/userlanguagemodel.h"
 #include "libime/pinyin/pinyincorrectionprofile.h"
 #include "libime/pinyin/pinyinencoder.h"
 #include "pinyindecoder.h"
-#include <cstddef>
-#include <fcitx-utils/connectableobject.h>
-#include <fcitx-utils/macros.h>
-#include <limits>
-#include <memory>
-#include <utility>
 
 namespace libime {
 
@@ -39,6 +39,7 @@ public:
     size_t beamSize_ = Decoder::beamSizeDefault;
     size_t frameSize_ = Decoder::frameSizeDefault;
     size_t partialLongWordLimit_ = 0;
+    size_t wordCandidateLimit_ = 15;
     float maxDistance_ = std::numeric_limits<float>::max();
     float minPath_ = -std::numeric_limits<float>::max();
     PinyinPreeditMode preeditMode_ = PinyinPreeditMode::RawText;
@@ -139,6 +140,19 @@ void PinyinIME::setPartialLongWordLimit(size_t n) {
     }
 }
 
+size_t PinyinIME::wordCandidateLimit() const {
+    FCITX_D();
+    return d->wordCandidateLimit_;
+}
+
+void PinyinIME::setWordCandidateLimit(size_t n) {
+    FCITX_D();
+    if (d->wordCandidateLimit_ != n) {
+        d->wordCandidateLimit_ = n;
+        emit<PinyinIME::optionChanged>();
+    }
+}
+
 void PinyinIME::setPreeditMode(PinyinPreeditMode mode) {
     FCITX_D();
     if (d->preeditMode_ != mode) {
